#ifndef CGAL_HELPERS_H
#define CGAL_HELPERS_H

#include "Types.h"

// Contains some helper functions for the curve classes. 

// binomials: n -> NumType *
// Parameters: 
//    n   a number
//    c   array of numbers with n+1 elements
// Effect: calculates the binomial coeficients
//         at the end c[i]=n!/i!(n-i)! for i in [0, n]
template <class NumType>
static void binomials(NumType * c, int n) {  
    c[0]=c[n]=1.0;
    for(int i=1; i<=(n/2); i++) {
	c[i] = (c[i-1]*(n-i+1))/i;
	c[n-i] = c[i];
    }
};

// in_tolerance: Point, Point, NumType -> bool
// Parameters: 
//    p         Point
//    q         Point
//    epsilon   the tolerance
// Returns: 
//    true      if the distance between p and q <= epsilon
// Template function
template <class NumType>
static bool in_tolerance(const CGAL::Point_2<CGAL::Cartesian<NumType> >& p, 
			 const CGAL::Point_2<CGAL::Cartesian<NumType> >& q, 
			 double epsilon=EPSILON) {
    NumType dist_2=(p-q)*(p-q);
    if(dist_2<=power(epsilon, 2)) return true;
    else return false;
};

// uniform_parametrisation: NumType, NumType, unsigned -> vector<NumType>
// Parameters:
//   param  vector<NumType> contains the uniform parametrisation over [a, b]
//          after the end
//   a      the lower bound of the interval
//   b      the upper bound of the interval
//   num    the number of points to be calculated in [a, b]
// Pre:
//   b>a
// Effect: fills the vector param with num uniform distributed values
//         over the interval [a, b]
template <class NumType>
static void uniform_parametrisation(std::vector<NumType>& param, NumType a, 
				    NumType b, unsigned num) {
    assert(b>a);
    param.resize(num);
    if(num>0) {
	param[0]=a;
	param[num-1] = b;
	if(num>2) {
	    NumType step=(b-a)/(num-1);
	    for(unsigned i=1; i<(num-1); i++) {
		param[i]=param[0] + i*step;
	    }
	}
    }      
};

// lin_subdivision: Points -> Points
// Parameters:
//    pts     the point sequence to be linear subdivided
//Effect:     after the execution contains pts the original points 
//            and theirs midpoints {...,p_i, (p_i+p_i+1)/2,...}
template <class NumType>
void lin_subdivision(std::vector<CGAL::Point_2<CGAL::Cartesian<NumType> > > & pts) {
    if(pts.empty() || pts.size()==1) return;
    int n = pts.size()-1;	
    std::vector<CGAL::Point_2<CGAL::Cartesian<NumType> > > tmp;
    tmp.push_back(pts[0]);
    for(int i=1; i<=n; i++) {
	tmp.push_back(midpoint(pts[i-1], pts[i]));
	tmp.push_back(pts[i]);
    }
    pts = tmp;
}

// averaging: Points -> Points
// Parameters:
//    pts     contains the point sequence to be average, after 
//            the computation contains the averaged sequence
template <class NumType>
void averaging(std::vector<CGAL::Point_2<CGAL::Cartesian<NumType> > > & pts) {
    if(pts.empty() || pts.size()==1) return;
    int n = pts.size()-1;
    std::vector<CGAL::Point_2<CGAL::Cartesian<NumType> > > tmp = pts;
    for(int i=1; i<=n; i++) {
	pts[i]=midpoint(tmp[i-1], tmp[i]);
    }
    pts.push_back(tmp[n]);
}

/*
      Diese Fkt. ersetzt die default Fkt. geo_object_dragging von GeoWin.
      Das Ziel ist auf die result-scene, die die Bezier-Kurve enthaelt nur
      ein mal update aufzurufen. Im Gegenteil wird die result_scene des 
      Kontrollpolygons staendig aktualisiert.
*/
// my_geo_object_dragging:  GeoWin, leda_point -> void
// Parameters:
//    gw   GepWin object
//    p    leda_point
// Effect: substitutes the default geo_object_dragging function of GeoWin so
//         that the update function for the result scene calls only efter
//         the mouse button releases
extern void my_geo_object_dragging(GeoWin& gw, const leda_point& p);

// link_points: Points -> SegmentList
// Parameters:
//   pts    the list of points to be linked 
//   segm   the resulting list of segments
// Effect: links the points by segments
template <class NumType>
void link_points_to_segments(const std::list<CGAL::Point_2<CGAL::Cartesian<NumType> > > &pts, 
	       std::list<CGAL::Segment_2<CGAL::Cartesian<NumType> > >& segm) {
    segm.clear();	
    if(pts.size() < 2) return;	    
    std::list<CGAL::Point_2<CGAL::Cartesian<NumType> > >::const_iterator it, next;
    it=pts.begin();
    next=it;
    next++;
    while(next!=pts.end()) {	   
	CGAL::Segment_2<CGAL::Cartesian<NumType> > s(*it, *next);
	segm.push_back(s);
	it = next;
	next++;
    }	
    cout<<endl;
}

// between_p_q: Point, Point, leda_rational -> Point
// Parameters:
//      p     Point
//      q     Point
//      m     the returned point pt = (1-m)*p + m*q
// Pre: 0<=m<=1
// Returns:  the point r on the line segment pq so that r = (1-m)*p + m*q    
CGAL::Point_2<CGAL::Cartesian<leda_rational> > 
   between_p_q(CGAL::Point_2<CGAL::Cartesian<leda_rational> >& p, 
	       CGAL::Point_2<CGAL::Cartesian<leda_rational> >& q, 
	       leda_rational m);

// between_p_q: Point, Point, NumType -> Point
// Parameters: 
//      p     Point
//      q     Point
//      m     the returned point pt = (1-m)*p + m*q
// Pre: 0<=m<=1
// Returns: pt = (1-m)*p + m*q
template <class NumType> 
CGAL::Point_2<CGAL::Cartesian<NumType> > 
   between_p_q(CGAL::Point_2<CGAL::Cartesian<NumType> >& p, 
	       CGAL::Point_2<CGAL::Cartesian<NumType> >& q, 
	       NumType m) {
    CGAL::Point_2<CGAL::Cartesian<NumType> > pt(((1-m)*p.x()+m*q.x()), 
					       ((1-m)*p.y()+m*q.y()));
	return pt;	
};

// _sqrt: leda_rational -> double
// Parameters:
//    number   a leda_rational number
// Returns : sqrt(number)
double _sqrt(leda_rational number);

// _sqrt: NumType -> double
// Parameters:
//    number   a number for wich the implicit conversion to double is defined
// Returns : sqrt(number)
template <class NumType>
double _sqrt(NumType number) {
    return sqrt(number);
};

// _to_float: leda_rational -> double
// Parameters:
//    number   a leda_rational number
// Returns : to_float(number) a double floating point wich approximates number
double _to_float(leda_rational number);

// _to_float: NumType -> double
// Parameters:
//    number   number types  for wich the implicit conversion 
//             to double is defined
// Returns : number
template <class NumType>
double _to_float(NumType number) {
    return (double) number;
};

#endif //CGAL_HELPERS_H
